#========================================================================================
# package      DE_EPAGES::Customer::t::unit::CustomerService
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  Unit test for DE_EPAGES::Customer::API::CustomerService.
#========================================================================================
package DE_EPAGES::Customer::t::unit::CustomerService;
use base DE_EPAGES::Core::API::Test;

use DE_EPAGES::Customer::API::CustomerService;
use DE_EPAGES::Test::Mock::MockifyEpagesObject;
use DE_EPAGES::Test::Mock::Mockify qw ( GetParametersFromMockifyCall );
use DE_EPAGES::Test::Mock::Mockery;
use Test::Exception;
use Test::More;

use strict;



sub test {
    my $self = shift;

    $self->test_getCustomers();
    $self->test_getCustomerData();

    $self->test_updateCustomerPartially();

    $self->test_createCustomer();
    $self->test_createCustomerWithSameCustomerNumber();

    done_testing();
}

sub test_getCustomers {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Mockery = $self->_createMocks();
    my $Shop = $Mockery->get('Shop');

    my $CustomerService = DE_EPAGES::Customer::API::CustomerService->new();
    my $hExpectedCustomerData = ['Customer1', 'Customer2'];

    is_deeply($CustomerService->getCustomers($Shop), $hExpectedCustomerData, "$SubTestName - test if customer array was returned");
}

sub test_getCustomerData {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Mockery = $self->_createMocks();
    my $Customer = $Mockery->get('Customer');

    my $CustomerService = DE_EPAGES::Customer::API::CustomerService->new();
    my $hCustomerData = $CustomerService->getCustomerData($Customer);

    my $hExpectedCustomerData = {
        'customerNumber' => 'TestCustomer123',
        'customerId' => '123-456-7890',
        'creationDate' => '2015-04-05T12:00:00Z',
        'internalNote' => 'internal note',
        'billingAddress' => {
            'company'=> 'billing company',
            'salutation'=> undef,
            'firstName'=> 'fname',
            'lastName'=> 'lname',
            'street'=> 'billstr 1',
            'streetDetails'=> undef,
            'zipCode'=> '100001',
            'city'=> 'onetown',
            'state'=> undef,
            'country'=> undef,
            'title'=> undef,
            'vatId'=> undef,
            'birthday'=> undef,
            'emailAddress'=> 'first.last@email.com',
        },
    };

    is_deeply($hCustomerData, $hExpectedCustomerData, "$SubTestName - test if expected customer data was returned.");
}



sub test_updateCustomerPartially {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Mockery = $self->_createMocks();

    my $Resource = $Mockery->get('SUT');
    my $Customer = $Mockery->get('Customer');

    my $hRequestBody_addCustomerNumber = {
        'op' => 'add',
        'path' => '/customerNumber',
        'value' => '123',
    };
    my $hRequestBody_removeInternalNote = {
        'op' => 'remove',
        'path' => '/internalNote',
    };

    my $ahRequestBody = [
        $hRequestBody_addCustomerNumber,
        $hRequestBody_removeInternalNote
    ];

    $Resource->updateCustomerPartially($Customer, $ahRequestBody);

    my $aPatchArgumentsCall1 = GetParametersFromMockifyCall($Mockery->get('CustomerPatch'), 'patch', 0);
    is_deeply($aPatchArgumentsCall1->[0], $hRequestBody_addCustomerNumber, "$SubTestName - test if patch was called with correct request data.");

    my $aPatchArgumentsCall2 = GetParametersFromMockifyCall($Mockery->get('CustomerPatch'), 'patch', 1);
    is_deeply($aPatchArgumentsCall2->[0], $hRequestBody_removeInternalNote, "$SubTestName - test if patch was called with correct request data.");
}



sub test_createCustomer {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Mockery = $self->_createMocks();
    my $CustomerService = $Mockery->get('SUT');
    my $Shop = $Mockery->get('ShopWithoutCustomer');

    my $hRequestBody = {
        'customerNumber' => 'TestCustomer123',
        'internalNote' => 'internal note',
        'billingAddress' => {
            'company'=> 'billing company',
            'salutation'=> undef,
            'firstName'=> 'fname',
            'lastName'=> 'lname',
            'street'=> 'billstr 1',
            'streetDetails'=> undef,
            'zipCode'=> '100001',
            'city'=> 'onetown',
            'state'=> undef,
            'country'=> undef,
            'title'=> undef,
            'vatId'=> undef,
            'birthday'=> undef,
            'emailAddress'=> 'first.last@email.com',
        }
    };
    my $hCustomerData = $CustomerService->createCustomer($Shop, $hRequestBody);

    my $hExpectedCustomerData = {
        'customerId' => '123-456-7890',
        'customerNumber' => 'TestCustomer123',
        'creationDate' => '2015-04-05T12:00:00Z',
        'internalNote' => 'internal note',
        'billingAddress' => {
            'company'=> 'billing company',
            'salutation'=> undef,
            'firstName'=> 'fname',
            'lastName'=> 'lname',
            'street'=> 'billstr 1',
            'streetDetails'=> undef,
            'zipCode'=> '100001',
            'city'=> 'onetown',
            'state'=> undef,
            'country'=> undef,
            'title'=> undef,
            'vatId'=> undef,
            'birthday'=> undef,
            'emailAddress'=> 'first.last@email.com',
        }
    };
    is_deeply($hCustomerData, $hExpectedCustomerData, "$SubTestName - test if expected customer data was returned.");
}



sub test_createCustomerWithSameCustomerNumber {
    my $self = shift;

    my $Mockery = $self->_createMocks();
    my $Shop = $Mockery->get('ShopWithCustomer');
    my $CustomerService = DE_EPAGES::Customer::API::CustomerService->new();

    my $hRequestBody = {
        'customerNumber' => 'TestCustomer123'
    };
    dies_ok{ $CustomerService->createCustomer($Shop, $hRequestBody)} 'Test passed with Error';

}



sub _createMocks {
    my $self = shift;

    my $Mockery = DE_EPAGES::Test::Mock::Mockery->new();

    $Mockery->register('SUT', ['CustomerPatch', 'SoapCustomerService'], sub {
        my ($CustomerPatch, $SoapCustomerService) = @_;
        my $Mockify = DE_EPAGES::Test::Mock::Mockify->new('DE_EPAGES::Customer::API::CustomerService', []);
        $Mockify->addMockWithReturnValueAndParameterCheck('_createCustomerPatch', $CustomerPatch, ['object']);
        $Mockify->addMockWithReturnValue('_createSOAPCustomerService', $SoapCustomerService);
        $Mockify->mock('_Transaction', sub {
            shift;
            my ($cCode, $cRollback) = @_;
            $cCode->();
        });
        return $Mockify->getMockObject();
    });

    $Mockery->register('CustomerPatch', [], sub {
        my $Mockify = DE_EPAGES::Test::Mock::Mockify->new('DE_EPAGES::Customer::API::CustomerPatch', []);
        $Mockify->addMockWithReturnValueAndParameterCheck('patch', undef, ['hashref']);
        return $Mockify->getMockObject();
    });

    $Mockery->register('Customer', ['Address'], sub {
        my ($Address) = @_;
        my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Customer::API::Object::Customer', {
            'Alias' => 'TestCustomer123',
            'GUID' => '123-456-7890',
            'CreationDate' => $self->_createDateTime(),
            'Comment' => 'internal note',
            'BillingAddress' => $Address,
        });
        return $Mockify->getMockObject();
    });

    $Mockery->register('ShopWithoutCustomer', ['CustomerClass', 'CustomerFolder'], sub {
        my ($CustomerClass, $CustomerFolder) = @_;
        my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Shop::API::Object::Shop', {
            'CustomerClass' => $CustomerClass,
        });
        $Mockify->addMockWithReturnValueAndParameterCheck('folder', $CustomerFolder, ['string']);
        return $Mockify->getMockObject();
    });

    $Mockery->register('ShopWithCustomer', ['CustomerClass', 'CustomerFolderWithCustomer'], sub {
        my ($CustomerClass, $CustomerFolder) = @_;
        my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Shop::API::Object::Shop', {
            'CustomerClass' => $CustomerClass,
        });
        $Mockify->addMockWithReturnValueAndParameterCheck('folder', $CustomerFolder, ['string']);
        return $Mockify->getMockObject();
    });

    $Mockery->register('CustomerClass', ['Customer'], sub {
        my ($Customer) = @_;
        my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Object::API::Object::Class', {});
        $Mockify->addMockWithReturnValueAndParameterCheck('insertObject', $Customer, ['hashref']);
        return $Mockify->getMockObject();
    });

    $Mockery->register('CustomerFolder', [], sub {
        my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Object::API::Object::Object', {});
        $Mockify->addMockWithReturnValueAndParameterCheck('existsChild', 0, ['string']);
        return $Mockify->getMockObject();
    });

    $Mockery->register('CustomerFolderWithCustomer', [], sub {
        my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Object::API::Object::Object', {});
        $Mockify->addMockWithReturnValueAndParameterCheck('existsChild', 1, ['string']);
        return $Mockify->getMockObject();
    });

    $Mockery->register('Shop', [], sub {
        my $hAttributes = {'Customers' => ['Customer1', 'Customer2']};
        my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Shop::API::Object::Shop', $hAttributes);
        return $Mockify->getMockObject();
    });

    $Mockery->register('SoapCustomerService', [], sub {
        my %Options = ('Object' => 1);
        my $Mock = DE_EPAGES::Test::Mock::Mockify->new('DE_EPAGES::Customer::API::WebService::CustomerService', [%Options]);
        $Mock->addMockWithReturnValueAndParameterCheck('setAddress', undef, ['object', 'hashref', 'string']);
        return $Mock->getMockObject();
    });

    $Mockery->register('Address', [], sub {
        my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Address::API::Object::Address', {
            'Company'=> 'billing company',
            'Salutation'=> undef,
            'FirstName'=> 'fname',
            'LastName'=> 'lname',
            'Street'=> 'billstr 1',
            'Street2'=> undef,
            'Zipcode'=> '100001',
            'City'=> 'onetown',
            'State'=> undef,
            'CountryID'=> undef,
            'Title'=> undef,
            'VATID'=> undef,
            'Birthday'=> undef,
            'EMail'=> 'first.last@email.com',
        });
        return $Mockify->getMockObject();
    });

    return $Mockery->assemble();
}



sub _createDateTime {
    my $self = shift;
    my $DateTime = DateTime->new(
        year       => 2015,
        month      => 4,
        day        => 5,
        hour       => 14,
        minute     => 0,
        second     => 0,
        nanosecond => 0,
        time_zone  => 'Europe/Berlin',
    );
    return $DateTime;
}

__PACKAGE__->runScript();