#========================================================================================
# package      DE_EPAGES::MeinPaket::t::unit::MeinPaketUtil
# base         DE_EPAGES::Test::API::UnitTest::EpagesUnitTest
#----------------------------------------------------------------------------------------
# description  Unit test for MeinPaketUtil
#========================================================================================
package DE_EPAGES::MeinPaket::t::unit::MeinPaketUtil;
use base DE_EPAGES::Test::API::UnitTest::Test;

use DE_EPAGES::MeinPaket::API::MeinPaketUtil;
use DE_EPAGES::MeinPaket::t::unit::MeinPaketMockFactory;
use DE_EPAGES::Test::Mock::Mockify;
use DE_EPAGES::MeinPaket::API::Constants qw ( %MEINPAKET_ONLINESTATUS );
use Test::More;

use strict;

sub test {
    my $self = shift;

    $self->prepareMocks();

    $self->test_wasProductUploaded();
    $self->test_isOfferOnline();
    $self->test_isLastUploadProductInOffer();
    $self->test_getOffersContainingProduct_ProductWithOffers();
    $self->test_getOffersContainingProduct_ProductWithoutOffers();
    $self->test_wasOfferUploaded();

    done_testing();
}

sub test_wasProductUploaded {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $UploadedProduct = $self->memory->get('UploadedProductA');
    my $NotUploadedProduct = $self->memory->get('NotUploadedProduct');

    my $SUT = DE_EPAGES::MeinPaket::API::MeinPaketUtil->new();

    is($SUT->wasProductUploaded($UploadedProduct), 1,
        "$SubTestName - test if product with MeinPaketId is considered uploaded.");
    is($SUT->wasProductUploaded($NotUploadedProduct), 0,
        "$SubTestName - test if product without MeinPaketId is not considered uploaded.");
}

sub test_isOfferOnline {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $MeinPaketOfferOnline = $self->memory->get('OfferOnline');
    my $MeinPaketOfferOffline = $self->memory->get('OfferOffline');

    my $SUT = DE_EPAGES::MeinPaket::API::MeinPaketUtil->new();

    is($SUT->isOfferOnline($MeinPaketOfferOnline), 1,
        "$SubTestName - test if product with MeinPaketId is considered uploaded.");
    is($SUT->isOfferOnline($MeinPaketOfferOffline), 0,
        "$SubTestName - test if product without MeinPaketId is not considered uploaded.");
}

sub test_isLastUploadProductInOffer {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Product = $self->memory->get('UploadedProductA');
    my $OfferOneUploadedProduct = $self->memory->get('MeinPaketOffer_OneUploadedProduct');
    my $OfferTwoUploadedProducts = $self->memory->get('MeinPaketOffer_TwoUploadedProducts');

    my $SUT = DE_EPAGES::MeinPaket::API::MeinPaketUtil->new();

    is($SUT->isLastUploadProductInOffer($Product, $OfferOneUploadedProduct), 1,
        "$SubTestName - test if offer containing only the passed product is evaluated true.");
    is($SUT->isLastUploadProductInOffer($Product, $OfferTwoUploadedProducts), 0,
        "$SubTestName - test if offer containing more than the passed product is evaluated false.");
}

sub test_getOffersContainingProduct_ProductWithOffers {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $SUT = $self->memory->get('SUT');
    my $MeinPaketClass = $self->memory->get('MeinPaketClass');

    my $Product = $self->memory->get('UploadedProductA');

    my $MeinPaketOfferOnline = $self->memory->get('OfferOnline');
    my $MeinPaketOfferOffline = $self->memory->get('OfferOffline');

    $MeinPaketClass->addMockWithReturnValueAndParameterCheck('getObjects', [$MeinPaketOfferOnline, $MeinPaketOfferOffline], [
        {'string' => 'ByProductID'},
        {'arrayref' => [ '123' ]},
    ]);
    $SUT->addMockWithReturnValueAndParameterCheck('_loadClassByAlias', $MeinPaketClass->getMockObject(), [
        {'string' => 'MeinPaketOffer'},
    ]);

    is_deeply($SUT->getMockObject()->getOffersContainingProductWithParent($Product),
        [$MeinPaketOfferOnline, $MeinPaketOfferOffline],
        "$SubTestName - test if product with offers returns the expected offers.");
}

sub test_getOffersContainingProduct_ProductWithoutOffers {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $SUT = $self->memory->get('SUT');
    my $MeinPaketClass = $self->memory->get('MeinPaketClass');
    my $Product = $self->memory->get('ProductWithoutOffer');

    $MeinPaketClass->addMockWithReturnValueAndParameterCheck('getObjects', [], [
        {'string' => 'ByProductID'},
        {'arrayref' => [ '123' ]},
    ]);
    $SUT->addMockWithReturnValueAndParameterCheck('_loadClassByAlias', $MeinPaketClass->getMockObject(), [
        {'string' => 'MeinPaketOffer'},
    ]);

    is_deeply($SUT->getMockObject()->getOffersContainingProductWithParent($Product),
        [], "$SubTestName - test if product without offers returns an empty list.");
}

sub test_wasOfferUploaded {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $OfferNoUploadedProducts = $self->memory->get('MeinPaketOffer_NoUploadedProducts');
    my $OfferTwoUploadedProducts = $self->memory->get('MeinPaketOffer_TwoUploadedProducts');

    my $SUT = DE_EPAGES::MeinPaket::API::MeinPaketUtil->new();

    is($SUT->wasOfferUploaded($OfferNoUploadedProducts), 0,
        "$SubTestName - test if offer containing no uploaded product is evaluated false.");
    is($SUT->wasOfferUploaded ($OfferTwoUploadedProducts), 1,
        "$SubTestName - test if offer containing at least one uploadet product is evaluated true.");
}

sub prepareMocks {
    my $self = shift;

    my $MockFactory = DE_EPAGES::MeinPaket::t::unit::MeinPaketMockFactory->new();
    $self->setFactory($MockFactory);

    my $UploadedProductA = $self->mockFactory->buildProduct({
        'ID' => '123',
        'Alias' => 'ABC-001',
        'SuperProduct' => undef,
        'MeinPaketID' => 'MeinPaket-001',
    })->getMockObject();

    my $UploadedProductB = $self->mockFactory->buildProduct({
        'Alias' => 'ABC-002',
        'MeinPaketID' => 'MeinPaket-002',
    })->getMockObject();

    my $NotUploadedProduct = $self->mockFactory->buildProduct({
        'Alias' => 'ABC-003',
        'MeinPaketID' => undef,
    })->getMockObject();

    my $ProductWithoutOffer = $self->mockFactory->buildProduct({
        'ID' => '123',
        'Alias' => 'XYZ-001',
        'SuperProduct' => undef,
        'MeinPaketID' => undef,
    })->getMockObject();

    $self->memory->add('UploadedProductA', $UploadedProductA);
    $self->memory->add('UploadedProductB', $UploadedProductB);
    $self->memory->add('NotUploadedProduct', $NotUploadedProduct);
    $self->memory->add('ProductWithoutOffer', $ProductWithoutOffer);

    my $MeinPaketOfferOnline = $self->mockFactory->buildMeinPaketOffer({
        'OnlineStatus' => $MEINPAKET_ONLINESTATUS{'OFFER_ONLINE'},
    })->getMockObject();
    my $MeinPaketOfferOffline = $self->mockFactory->buildMeinPaketOffer({
        'OnlineStatus' => $MEINPAKET_ONLINESTATUS{'OFFER_OFFLINE'},
    })->getMockObject();

    $self->memory->add('OfferOnline', $MeinPaketOfferOnline);
    $self->memory->add('OfferOffline', $MeinPaketOfferOffline);

    my $MeinPaketOfferNoUploadedProducts = $self->mockFactory->buildMeinPaketOffer({});
    $MeinPaketOfferNoUploadedProducts->addMockWithReturnValue('getProducts',
        [$NotUploadedProduct]);

    my $MeinPaketOfferOneUploadedProduct = $self->mockFactory->buildMeinPaketOffer({});
    $MeinPaketOfferOneUploadedProduct->addMockWithReturnValue('getProducts',
        [$UploadedProductA, $NotUploadedProduct]);

    my $MeinPaketOfferTwoUploadedProducts = $self->mockFactory->buildMeinPaketOffer({});
    $MeinPaketOfferTwoUploadedProducts->addMockWithReturnValue('getProducts',
        [$UploadedProductA, $UploadedProductB, $NotUploadedProduct]);

    $self->memory->add('MeinPaketOffer_NoUploadedProducts', $MeinPaketOfferNoUploadedProducts->getMockObject());
    $self->memory->add('MeinPaketOffer_OneUploadedProduct', $MeinPaketOfferOneUploadedProduct->getMockObject());
    $self->memory->add('MeinPaketOffer_TwoUploadedProducts', $MeinPaketOfferTwoUploadedProducts->getMockObject());

    my $MeinPaketClass = $self->mockFactory->buildClass({});
    $self->memory->add('MeinPaketClass', $MeinPaketClass);

    my $SUT = DE_EPAGES::Test::Mock::Mockify->new('DE_EPAGES::MeinPaket::API::MeinPaketUtil', []);
    $self->memory->add('SUT', $SUT);
}

__PACKAGE__->runScript();